@@ -25,44 +25,116 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include <ctype.h>
 #include <unistd.h>
 #include <hidapi/hidapi.h>
+#include <argp.h>
 #include "libusbrelay.h"
 #include "usbrelay.h"
+#include "gitversion.h"
 
 #define MIN(a, b) ((a) < (b) ? (a) : (b))
 
+static void argp_print_version(FILE *stream, struct argp_state *state)
+{
+	fprintf(stream, "libusbrelay: %s\nusbrelay: %s\n", libusbrelay_version(), GITVERSION);
+}
+
+void (*argp_program_version_hook)(FILE *stream, struct argp_state *state) = argp_print_version;
+
+const char *argp_program_bug_address =
+	"https://github.com/darrylb123/usbrelay/issues";
+
+/* Program documentation. */
+static char doc[] =
+	"Control or query USB HID relays."
+
+	/* This part of the documentation comes after the options */
+	"\vWithout ACTION, the actual state of all relays is printed to stdout.\n"
+	"ACTION can be one of:\n"
+	"RELID_N=[0|1] to switch the N-th relay off or on\n"
+	"RELID_0=NEWID to change relay ID\n"
+	;
+
+/* A description of the arguments we accept. */
+static char args_doc[] = "[ACTION...]";
+
+/* The options we understand. */
+static struct argp_option options[] = {
+	{"debug",    'd', 0,       0, "Produce debugging output" },
+	{"quiet",    'q', 0,       0, "Be quiet" },
+	{"export-id",'e', "DEV",   0, "Print relay ID of the given DEV (/dev/hidrawXX) in udev-compatible format" },
+	{ 0 }
+};
+
+/* Used by ‘main’ to communicate with ‘parse_opt’. */
+struct arguments
+{
+	int debug;
+	int verbose;
+	char *export_id;	/* "/dev/hidrawXX" or NULL */
+};
+
+/* Parse a single option. */
+static error_t
+parse_opt (int key, char *arg, struct argp_state *state)
+{
+	/* Get the ‘input’ argument from ‘argp_parse’, which we
+	   know is a pointer to our arguments structure. */
+	struct arguments *args = state->input;
+
+	switch (key) {
+	case 'd':
+		args->debug = 1;
+		break;
+	case 'e':
+		args->export_id = arg;
+		break;
+	case 'q':
+		args->verbose = 0;
+		break;
+
+	case ARGP_KEY_NO_ARGS:
+		break;
+
+	default:
+		return ARGP_ERR_UNKNOWN;
+	}
+	return 0;
+}
+
+/* Our argp parser. */
+static struct argp argp = { options, parse_opt, args_doc, doc };
+
+
+
 int main(int argc, char *argv[])
 {
 	struct relay *relays = 0;
-	int debug = 0;
-	int verbose = 1;
-	int i,c;
+	int i;
 	int exit_code = 0;
+	struct arguments args = {
+		.debug = 0,
+		.export_id = NULL,
+		.verbose = 1,
+	};
+
+       /* Parse our arguments; every option seen by ‘parse_opt’ will
+          be reflected in ‘args’. */
+       argp_parse (&argp, argc, argv, 0, &optind, &args);
 
-	while ((c = getopt (argc, argv, "qd")) != -1) {
-    switch (c) {
-      case 'q':
-        verbose = 0;
-        break;
-      case 'd':
-        debug = 1;
-        break;
-      }
-	}
 	/* allocate the memory for all the relays */
-	if ((argc - optind) > 1) {
+	if ((argc - optind) >= 1) {
 		relays = calloc(argc - optind + 1, sizeof(struct relay));	/* Yeah, I know. Not using the first member */
 		/* relays is zero-initialized */
 	} 
 	/* loop through the command line and grab the relay details */
 	for (i = 0 ; i < (argc - optind); i++) {
-	   printf("argv %s\n",argv[i + optind]);
 		char *arg = argv[i + optind];
 		struct relay *relay = &relays[i];
 
 		char *underscore = strchr(arg, '_');
+		char *second_underscore = strrchr(arg, '_'); /* Search from end of string too. Should be only one _ or = */
 		char *equal_sign = strchr(arg, '=');
-
-		if (underscore && equal_sign && underscore > equal_sign) {	/* e.g. ASDFG=QWERT_1 */
+		char *second_equals = strrchr(arg, '=');
+		if (!underscore || !equal_sign || (underscore > equal_sign) || (equal_sign != second_equals) || (underscore != second_underscore)){	/* e.g. ASDFG=QWERT_1 */
 			fprintf(stderr, "Invalid relay specification: %s\n",
 				argv[i + optind]);
 			exit(1);
@@ -78,6 +150,7 @@ int main(int argc, char *argv[])
 
 		relay->this_serial = malloc(size + 1);
 		strncpy(relay->this_serial, arg, size);
+		relay->this_serial[size] = 0;
 
 		/* Parse relay number */
 		if (underscore)
@@ -87,6 +160,7 @@ int main(int argc, char *argv[])
 			if (relay->relay_num == 0) {	/* command to change the serial - remaining token is the new serial */
 				strncpy(relay->new_serial, equal_sign + 1,
 					sizeof(relay->new_serial) - 1);
+				relay->new_serial[sizeof(relay->new_serial) -1] = 0;
 			} else {
 				if (atoi(equal_sign + 1)) {
 					relays[i].state = CMD_ON;
@@ -95,7 +169,7 @@ int main(int argc, char *argv[])
 				}
 			}
 		}
-		if (debug) {
+		if (args.debug) {
 			fprintf(stderr, "Orig: %s, Serial: %s, Relay: %d State: %x\n",
 				arg, relay->this_serial, 
 				relay->relay_num,
@@ -103,22 +177,38 @@ int main(int argc, char *argv[])
 			relay->found = 0;
 		}
 	}
-
+	// Warn if library is different version
+	if( strcmp(libusbrelay_version(),GITVERSION)) {
+		fprintf(stderr, "Warning: Version difference\nlibusbrelay: %s\nusbrelay: %s\n", libusbrelay_version(), GITVERSION);
+	}
+	
 	//Locate and identify attached relay boards
-	enumerate_relay_boards(getenv("USBID"), verbose, debug);
+	if(args.debug) 
+		fprintf(stderr, "libusbrelay: %s\nusbrelay: %s\n", libusbrelay_version(), GITVERSION);
+	enumerate_relay_boards(getenv("USBID"), args.verbose, args.debug);
+
+	if (args.export_id) {
+		relay_board *relay = find_board(args.export_id, args.debug);
+		if (!relay) {
+			fprintf(stderr, "relay '%s' not found\n", args.export_id);
+			exit(1);
+		}
+		printf("ID_SERIAL=%s\n", relay->serial);
+		return 0;
+	}
 
 	/* loop through the supplied command line and try to match the serial */
 	for (i = 0; i < (argc - optind); i++) {
-		if (debug ) {
-			fprintf(stderr, "Serial: %s, Relay: %d State: %x \n",
-				relays[i].this_serial, 
+		if (args.debug ) {
+			fprintf(stderr, "main() arg %d Serial: %s, Relay: %d State: %x \n",
+				i,relays[i].this_serial, 
 				relays[i].relay_num,
 				relays[i].state);
 		}
-		relay_board *board = find_board(relays[i].this_serial);
+		relay_board *board = find_board(relays[i].this_serial, args.debug );
 
 		if (board) {
-			if (debug == 2)
+			if (args.debug == 2)
 				fprintf(stderr, "%d HID Serial: %s ", i, board->serial);
 			if (relays[i].relay_num == 0) {
 				if (!relays[i].new_serial[0]) {
@@ -126,16 +216,16 @@ int main(int argc, char *argv[])
 				} else {
 					fprintf(stderr, "Setting new serial\n");
 					set_serial(board->serial,
-						   relays[i].new_serial);
+						   relays[i].new_serial,args.debug);
 				}
 			} else {
-				if (debug)
+				if (args.debug)
 					fprintf(stderr,
-						"Serial: %s, Relay: %d State: %x\n",
+						"main() operate: %s, Relay: %d State: %x\n",
 						relays[i].this_serial,
 						relays[i].relay_num,
 						relays[i].state);
-				if (operate_relay(board->serial, relays[i].relay_num,relays[i].state) < 0)
+				if (operate_relay(relays[i].this_serial, relays[i].relay_num,relays[i].state,args.debug) < 0)
 					exit_code++;
 				relays[i].found = 1;
 			}
@@ -145,21 +235,21 @@ int main(int argc, char *argv[])
 	shutdown();
 
 	for (i = 1; i < (argc - optind); i++) {
-		if (debug)
+		if (args.debug)
 			fprintf(stderr,"Serial: %s, Relay: %d State: %x ",
 				relays[i].this_serial, relays[i].relay_num,
 				relays[i].state);
 		if (relays[i].found) {
-			if (debug)
+			if (args.debug)
 				fprintf(stderr, "--- Found\n");
 		} else {
-			if (debug)
+			if (args.debug)
 				fprintf(stderr, "--- Not Found\n");
 			exit_code++;
 		}
 	}
 	if (relays) {
-		for (i = 1; i < (argc - optind) ; i++) {
+		for (i = 0; i < (argc - optind) ; i++) {
 			free(relays[i].this_serial);
 		}
 		free(relays);
